# -*- coding: utf-8 -*-

from datetime import datetime

from .media import _media_upload
from .type import RequestType


def complaint_list_query(self, begin_date=None, end_date=None, limit=10, offset=0, complainted_mchid=None):
    """查询投诉单列表
    :param begin_date: 开始日期，投诉发生的开始日期，格式为YYYY-MM-DD。注意，查询日期跨度不超过30天，当前查询为实时查询。示例值:'2019-01-01'
    :param end_date: 结束日期，投诉发生的结束日期，格式为YYYY-MM-DD。注意，查询日期跨度不超过30天，当前查询为实时查询。示例值:'2019-01-01'
    :param limit: 分页大小，设置该次请求返回的最大投诉条数，范围【1,50】,商户自定义字段，不传默认为10。示例值:5
    :param offset: 分页开始位置，该次请求的分页开始位置，从0开始计数，例如offset=10，表示从第11条记录开始返回，不传默认为0 。示例值:10
    :param complainted_mchid: 被诉商户号，投诉单对应的被诉商户号。示例值:'1900012181'
    """
    if not begin_date:
        begin_date = datetime.now().strftime("%Y-%m-%d")
    if not end_date:
        end_date = begin_date
    if not complainted_mchid:
        complainted_mchid = self._mchid
    path = '/v3/merchant-service/complaints-v2?limit=%s&offset=%s&begin_date=%s&end_date=%s&complainted_mchid=%s'
    path = path % (limit, offset, begin_date, end_date, complainted_mchid)
    return self._core.request(path)


def complaint_detail_query(self, complaint_id):
    """查询投诉单详情
    :param complaint_id: 投诉单对应的投诉单号。示例值:'200201820200101080076610000'
    """
    if not complaint_id:
        raise Exception('complaint_id is not assigned.')
    path = '/v3/merchant-service/complaints-v2/%s' % complaint_id
    return self._core.request(path)


def complaint_history_query(self, complaint_id, limit=100, offset=0):
    """查询投诉协商历史
    :param complaint_id: 投诉单对应的投诉单号。示例值:'200201820200101080076610000'
    :param limit: 分页大小，设置该次请求返回的最大协商历史条数，范围[1,300]，不传默认为100。。示例值:5
    :param offset: 分页开始位置，该次请求的分页开始位置，从0开始计数，例如offset=10，表示从第11条记录开始返回，不传默认为0。示例值:10
    """
    if not complaint_id:
        raise Exception('complaint_id is not assigned.')
    if limit not in range(1, 301):
        limit = 100
    path = '/v3/merchant-service/complaints-v2/%s/negotiation-historys?limit=%s&offset=%s' % (complaint_id, limit, offset)
    return self._core.request(path)


def complaint_notification_create(self, url):
    """创建投诉通知回调地址
    :param: url: 通知地址，仅支持https。示例值:'https://www.xxx.com/notify'
    """
    params = {}
    if url:
        params.update({'url': url})
    else:
        raise Exception('url is not assigned.')
    path = '/v3/merchant-service/complaint-notifications'
    return self._core.request(path, method=RequestType.POST, data=params)


def complaint_notification_query(self):
    """查询投诉通知回调地址
    :param: url: 通知地址，仅支持https。示例值:'https://www.xxx.com/notify'
    """
    path = '/v3/merchant-service/complaint-notifications'
    return self._core.request(path)


def complaint_notification_update(self, url):
    """更新投诉通知回调地址
    :param: url: 通知地址，仅支持https。示例值:'https://www.xxx.com/notify'
    """
    params = {}
    if url:
        params.update({'url': url})
    else:
        raise Exception('url is not assigned.')
    path = '/v3/merchant-service/complaint-notifications'
    return self._core.request(path, method=RequestType.PUT, data=params)


def complaint_notification_delete(self):
    """删除投诉通知回调地址
    :param: url: 通知地址，仅支持https。示例值:'https://www.xxx.com/notify'
    """
    path = '/v3/merchant-service/complaint-notifications'
    return self._core.request(path, method=RequestType.DELETE)


def complaint_response(self, complaint_id, response_content, response_images=None, jump_url=None, jump_url_text=None):
    """提交投诉回复
    :param complaint_id: 投诉单对应的投诉单号。示例值:'200201820200101080076610000'
    :param response_content: 回复内容，具体的投诉处理方案，限制200个字符以内。示例值:'已与用户沟通解决'
    :param response_images: 回复图片，传入调用商户上传反馈图片接口返回的media_id，最多上传4张图片凭证。示例值:['file23578_21798531.jpg', 'file23578_21798532.jpg']
    :param jump_url: 跳转链接，附加跳转链接，引导用户跳转至商户客诉处理页面，链接需满足https格式。示例值:"https://www.xxx.com/notify"
    :param jump_url_text: 转链接文案，展示给用户的文案，附在回复内容之后。用户点击文案，即可进行跳转。示例值:"查看订单详情"
    """
    params = {}
    if not complaint_id:
        raise Exception('complaint_id is not assigned')
    if response_content:
        params.update({'response_content': response_content})
    else:
        raise Exception('response_content is not assigned')
    params.update({'complainted_mchid': self._core._mchid})
    if response_images:
        params.update({'response_images': response_images})
    if jump_url:
        params.update({'jump_url': jump_url})
    if jump_url_text:
        params.update({'jump_url_text': jump_url_text})
    path = '/v3/merchant-service/complaints-v2/%s/response' % complaint_id
    return self._core.request(path, method=RequestType.POST, data=params)


def complaint_complete(self, complaint_id):
    """反馈投诉处理完成
    :param complaint_id: 投诉单对应的投诉单号。示例值:'200201820200101080076610000'
    """
    params = {}
    if not complaint_id:
        raise Exception('complaint_id is not assigned')
    params.update({'complainted_mchid': self._core._mchid})
    path = '/v3/merchant-service/complaints-v2/%s/complete' % complaint_id
    return self._core.request(path, method=RequestType.POST, data=params)


def complaint_image_upload(self, filepath, filename=None):
    """商户上传投诉反馈图片
    :param filepath: 图片文件路径
    :param filename: 文件名称，未指定则从filepath参数中截取
    """
    return _media_upload(self, filepath, filename, '/v3/merchant-service/images/upload')


def complaint_image_download(self, media_url):
    """下载客户投诉图片
    :param media_url: 图片下载地址，示例值:'https://api.mch.weixin.qq.com/v3/merchant-service/images/xxxxx'
    """
    path = media_url[len(self._core._gate_way):] if media_url.startswith(self._core._gate_way) else media_url
    return self._core.request(path, skip_verify=True)


def complaint_update_refund(self, complaint_id, action, launch_refund_day=None, reject_reason=None, reject_media_list={}, remark=None):
    """更新退款审批结果
    :param compaint_id: 投诉单对应的投诉单号。示例值:'200201820200101080076610000'
    :param action: 审批动作，同意 或 拒绝，REJECT：拒绝，拒绝退款；APPROVE：同意，同意退款；示例值：'APPROVE'
    :param launch_refund_day: 预计发起退款时间，预计将在多少个工作日内能发起退款, 0代表当天。示例值：3
    :param reject_reason: 拒绝退款原因，示例值：'拒绝退款'
    :param reject_media_list: 拒绝退款的举证图片列表，传入调用“商户上传反馈图片”接口返回的media_id，最多上传4张图片凭证，示例值：{'file23578_21798531.jpg'}
    :param remark: 备注，示例值：'已处理完成'
    """
    if complaint_id:
        path = '/v3/merchant-service/complaints-v2/%s/update-refund-progress' % complaint_id
    else:
        raise Exception('complaint_id is not assigned')
    params = {}
    if action:
        params.update({'action': action})
    else:
        raise Exception('action is not assigned')
    if isinstance(launch_refund_day, int):
        params.update({'launch_refund_day': launch_refund_day})
    if reject_reason:
        params.update({'reject_reason': reject_reason})
    if reject_media_list:
        params.update({'reject_media_list': reject_media_list})
    if remark:
        params.update({'remark': remark})
    return self._core.request(path, method=RequestType.POST, data=params)



# -*- coding: utf-8 -*-

from .type import RequestType


def goldplan_plan_change(self, sub_mchid, operation_type):
    """点金计划管理
    :param sub_mchid: 子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    :param operation_type: 操作类型, 枚举值:'OPEN':表示开通点金计划，'CLOSE':表示关闭点金计划。示例值:'OPEN'
    """
    params = {}
    if sub_mchid:
        params.update({'sub_mchid': sub_mchid})
    else:
        raise Exception('sub_mchid is not assigned.')
    if operation_type:
        params.update({'operation_type': operation_type})
    else:
        raise Exception('operation_type is not assigned.')
    path = '/v3/goldplan/merchants/changegoldplanstatus'
    return self._core.request(path, method=RequestType.POST, data=params)


def goldplan_custompage_change(self, sub_mchid, operation_type):
    """商家小票管理
    :param sub_mchid: 子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    :param operation_type: 操作类型, 枚举值:'OPEN':表示开通商家自定义小票，'CLOSE':表示关闭商家自定义小票。示例值:'OPEN'
    """
    params = {}
    if sub_mchid:
        params.update({'sub_mchid': sub_mchid})
    else:
        raise Exception('sub_mchid is not assigned.')
    if operation_type:
        params.update({'operation_type': operation_type})
    else:
        raise Exception('operation_type is not assigned.')
    path = '/v3/goldplan/merchants/changecustompagestatus'
    return self._core.request(path, method=RequestType.POST, data=params)


def goldplan_advertising_filter(self, sub_mchid, advertising_industry_filters):
    """同业过滤标签管理
    :param sub_mchid: 子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    :param advertising_industry_filters: 同业过滤标签值, 同业过滤标签最少传一个，最多三个。示例值:['SOFTWARE','SECURITY','LOVE_MARRIAGE']
    """
    params = {}
    if sub_mchid:
        params.update({'sub_mchid': sub_mchid})
    else:
        raise Exception('sub_mchid is not assigned.')
    if advertising_industry_filters:
        params.update({'advertising_industry_filters': advertising_industry_filters})
    else:
        raise Exception('advertising_industry_filters is not assigned.')
    path = '/v3/goldplan/merchants/set-advertising-industry-filter'
    return self._core.request(path, method=RequestType.POST, data=params)


def goldplan_advertising_open(self, sub_mchid, advertising_industry_filters=None):
    """开通广告展示
    :param sub_mchid: 子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    :param advertising_industry_filters: 同业过滤标签值, 同业过滤标签最少传一个，最多三个。示例值:['SOFTWARE','SECURITY','LOVE_MARRIAGE']
    """
    params = {}
    if sub_mchid:
        params.update({'sub_mchid': sub_mchid})
    else:
        raise Exception('sub_mchid is not assigned.')
    if advertising_industry_filters:
        params.update({'advertising_industry_filters': advertising_industry_filters})
    else:
        raise Exception('advertising_industry_filters is not assigned.')
    path = '/v3/goldplan/merchants/open-advertising-show'
    return self._core.request(path, method=RequestType.POST, data=params)


def goldplan_advertising_close(self, sub_mchid):
    """关闭广告展示
    :param sub_mchid: 子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    params = {}
    if sub_mchid:
        params.update({'sub_mchid': sub_mchid})
    else:
        raise Exception('sub_mchid is not assigned.')
    path = '/v3/goldplan/merchants/close-advertising-show'
    return self._core.request(path, method=RequestType.POST, data=params)



# -*- coding: utf-8 -*-

from .type import RequestType


def points_notify(self, transaction_id, openid, earn_points, increased_points, points_update_time, no_points_remarks=None, total_points=None, appid=None, sub_mchid=None):
    """智慧商圈积分同步
    :param transaction_id: 微信订单号，示例值:'1217752501201407033233368018'
    :param openid: 用户标识，示例值:'oWmnN4xxxxxxxxxxe92NHIGf1xd8'
    :param earn_points: 是否获得积分，示例值:True
    :param increased_points: 订单新增积分值，示例值:100
    :param points_update_time: 积分更新时间，示例值:'2020-05-20T13:29:35.120+08:00'
    :param no_points_remarks: 未获得积分的备注信息，示例值:'商品不参与积分活动'
    :param total_points: 顾客积分总额，示例值:888888
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    params = {}
    params.update({'appid': appid or self._appid})
    if transaction_id:
        params.update({'transaction_id': transaction_id})
    else:
        raise Exception('transaction_id is not assigned.')
    if openid:
        params.update({'openid': openid})
    else:
        raise Exception('openid is not assigned.')
    if earn_points:
        params.update({'earn_points': earn_points})
    else:
        raise Exception('earn_points is not assigned.')
    if increased_points:
        params.update({'increased_points': increased_points})
    else:
        raise Exception('increased_points is not assigned')
    if points_update_time:
        params.update({'points_update_time': points_update_time})
    else:
        raise Exception('points_update_time is not assigned.')
    if no_points_remarks:
        params.update({'no_points_remarks': no_points_remarks})
    if total_points:
        params.update({'total_points': total_points})
    if self._partner_mode and sub_mchid:
        params.update({'sub_mchid': sub_mchid})
    path = '/v3/businesscircle/points/notify'
    return self._core.request(path, method=RequestType.POST, data=params)


def user_authorization(self, openid, appid=None, sub_mchid=None):
    """智慧商圈积分授权查询
    :param openid: 用户标识，示例值:'oWmnN4xxxxxxxxxxe92NHIGf1xd8'
    :param appid: 小程序appid，顾客授权积分时使用的小程序的appid，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    if openid:
        if self._partner_mode:
            path = '/v3/businesscircle/user-authorizations/%s?appid=%s' % (openid, appid or self._appid)
            if sub_mchid:
                path = '%s&sub_mchid=%s' % (path, sub_mchid)
        else:
            path = '/v3/businesscircle/user-authorizations/%s?appid=%s' % (openid, self._appid)
    else:
        raise Exception('openid is not assigned.')
    return self._core.request(path)


def business_parking_sync(self, openid, brandid, plate_number, state, time, appid=None, sub_mchid=None):
    """商圈会员停车状态同步
    :param openid: 用户标识，示例值:'oWmnN4xxxxxxxxxxe92NHIGf1xd8'
    :param brandid: 品牌ID，示例值:1000
    :param plate_number: 车牌号，示例值: '粤B888888'
    :param state: 停车状态，IN=入场，用户开车进入商圈，OUT=离场，用户开车离开商圈。示例值:IN
    :param time: 时间，示例值：2022-06-01T10:43:39+08:00
    :param appid: 小程序appid，顾客授权积分时使用的小程序的appid，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    params = {}
    params.update({'appid': appid or self._appid})
    if not openid:
        raise Exception('openid is not assigned.')
    else:
        params.update({'openid': openid})
    if not brandid:
        raise Exception('brandid is not assigned.')
    else:
        params.update({'brandid': brandid})
    if not plate_number:
        raise Exception('plate_number is not assigned.')
    else:
        params.update({'plate_number': plate_number})
    if not state:
        raise Exception('state is not assigned.')
    else:
        params.update({'state': state})
    if not time:
        raise Exception('time is not assigned.')
    else:
        params.update({'time': time})
    if self._partner_mode:
        if not sub_mchid:
            raise Exception('sub_mchid is not assigned.')
        else:
            params.update({'sub_mchid': sub_mchid})
    path = 'https://api.mch.weixin.qq.com/v3/businesscircle/parkings'
    return self._core.request(path, method=RequestType.POST, date=params)


def business_point_status(self, openid, brandid, appid=None, sub_mchid=None):
    """商圈会员待积分状态查询
    :param openid: 用户标识，示例值:'oWmnN4xxxxxxxxxxe92NHIGf1xd8'
    :param brandid: 品牌ID，示例值:1000
    :param appid: 小程序appid，顾客授权积分时使用的小程序的appid，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    if not (openid and brandid):
        raise Exception('openid and/or brandid is not assigned.')
    else:
        path = 'https://api.mch.weixin.qq.com/v3/businesscircle/users/%s/points/commit_status?brandid=%s&appid=%s' % (openid, brandid, appid or self._appid)
    if sub_mchid:
        path += '%s&sub_mchid=%s' % (path, sub_mchid)
    return self._core.request(path)



from setuptools import setup


with open("README.md", "r", encoding="utf8") as f:
    long_description = f.read()

setup(
    name="wechatpayv3",
    version="1.2.36",
    author="minibear",
    description="微信支付 API v3 Python SDK(python sdk for wechatpay v3)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    keywords="python sdk wechatpay api v3 微信支付",
    url="https://github.com/minibear2021/wechatpayv3",
    packages=["wechatpayv3"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Topic :: Office/Business :: Financial",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
    install_requires=["requests>=2.21.0", "cryptography>=35.0.0"],
)



# -*- coding: utf-8 -*-
import os.path

from .type import RequestType
from .utils import sm3

# https://pay.weixin.qq.com/wiki/doc/apiv3/Offline/open/chapter4_8_1.shtml


def fapiao_card_template(self, card_template_information, card_appid=None):
    """创建电子发票卡券模板
    :param card_template_information: 卡券模板信息。示例值:{'logo_url':'http://mmbiz.qpic.cn/mmbiz/iaL1LJM1mF9aRKPZJkmG8xX'}
    :param card_appid: 插卡公众号AppID，若是服务商模式，则可以是服务商申请的appid，也可以是子商户申请的appid；若是直连模式，则是直连商户申请的appid。示例值：wxb1170446a4c0a5a2
    """
    if not card_appid:
        card_appid = self._appid
    params = {}
    params.update({'card_appid': card_appid})
    if card_template_information:
        params.update({'card_template_information': card_template_information})
    else:
        raise Exception('card_template_information is not assigned.')
    path = '/v3/new-tax-control-fapiao/card-template'
    return self._core.request(path, method=RequestType.POST, data=params)


def fapiao_set_merchant_config(self, callback_url=None):
    """配置开发选项
    :param callback_url: 商户回调地址。收取微信的授权通知、开票通知、插卡通知等相关通知。示例值：'https://pay.weixin.qq.com/callback'
    """
    if not callback_url:
        callback_url = self._notify_url
    params = {}
    params.update({'callback_url': callback_url})
    path = '/v3/new-tax-control-fapiao/merchant/development-config'
    return self._core.request(path, method=RequestType.POST, data=params)


def fapiao_merchant_config(self):
    """查询商户配置的开发选项
    """
    path = '/v3/new-tax-control-fapiao/merchant/development-config'
    return self._core.request(path)


def fapiao_title_url(self, fapiao_apply_id, source, total_amount, openid, appid=None,
                     seller_name=None, show_phone_cell=False, must_input_phone=False,
                     show_email_cell=False, must_input_email=False):
    """获取抬头填写链接
    :param fapiao_apply_id: 发票申请单号，示例值：'4200000444201910177461284488'
    :param source: 开票来源，WEB：微信H5开票，MINIPROGRAM：微信小程序开票，示例值：'WEB'
    :param total_amount: 总金额，单位：分，示例值：100
    :param openid: 需要填写发票抬头的用户在商户AppID下的OpenID，示例值：'plN5twRbHym_j-QcqCzstl0HmwEs'
    :param appid: 若开票来源是WEB，则为商户的公众号AppID；若开票来源是MINIPROGRAM，则为商户的小程序AppID，示例值：'wxb1170446a4c0a5a2'
    :param seller_name: 销售方名称，若不传则默认取商户名称，示例值：'深圳市南山区测试商户'
    :param show_phone_cell: 是否需要展示手机号填写栏
    :param must_input_phone: 是否必须填写手机号，仅当需要展示手机号填写栏时生效
    :param show_email_cell: 是否需要展示邮箱地址填写栏
    :param must_input_email: 是否必须填写邮箱地址，仅当需要展示邮箱地址填写栏时生效
    """
    path = '/v3/new-tax-control-fapiao/user-title/title-url?'
    if fapiao_apply_id:
        path += 'fapiao_apply_id=%s' % fapiao_apply_id
    else:
        raise Exception('fapiao_apply_id is not assigned.')
    if source:
        path += '&source=%s' % source
    else:
        raise Exception('source is not assigned.')
    if total_amount:
        path += '&total_amount=%s' % total_amount
    else:
        raise Exception('total_amount is not assigned.')
    if appid:
        path += '&appid=%s' % appid
    else:
        path += '&appid=%s' % self._appid
    if openid:
        path += '&openid=%s' % openid
    else:
        raise Exception('openid is not assigned.')
    if seller_name:
        path += '&seller_name=%s' % seller_name
    if show_phone_cell:
        path += '&show_phone_cell=true'
    if must_input_phone:
        path += '&must_input_phone=true'
    if show_email_cell:
        path += '&show_email_cell=true'
    if must_input_email:
        path += '&must_input_email=true'
    return self._core.request(path)


def fapiao_title(self, fapiao_apply_id, scene='WITH_WECHATPAY'):
    """获取用户填写的抬头
    :param fapiao_apply_id: 发票申请单号，示例值：'4200000444201910177461284488'
    :param scene: 场景值，目前只支持WITH_WECHATPAY。示例值：'WITH_WECHATPAY'
    """
    path = '/v3/new-tax-control-fapiao/user-title'
    if fapiao_apply_id:
        path += 'fapiao_apply_id=%s' % fapiao_apply_id
    else:
        raise Exception('fapiao_apply_id is not assigned.')
    path += '&scene=%s' % scene
    return self._core.request(path)


def fapiao_tax_codes(self, offset=0, limit=20):
    """获取商品和服务税收分类对照表
    :param offset: 查询的起始位置，示例值：0
    :param limit: 查询的最大数量，最大值20
    """
    path = '/v3/new-tax-control-fapiao/merchant/tax-codes?offset=%s&limit=%s' % (offset, limit)
    return self._core.request(path)


def fapiao_merchant_base_info(self):
    """获取商户开票基础信息
    """
    path = '/v3/new-tax-control-fapiao/merchant/base-information'
    return self._core.request(path)


def fapiao_applications(self, fapiao_apply_id, buyer_information, fapiao_information, scene='WITH_WECHATPAY'):
    """开具电子发票
    :param fapiao_apply_id: 发票申请单号，示例值：'4200000444201910177461284488'
    :param buyer_information: 购买方信息，示例值：{'type':'ORGANIZATION','name':'深圳市南山区测试企业'}
    :param fapiao_information: 需要开具的发票信息，示例值：[{'fapiao_id':'20200701123456','total_amount':382895,'need_list':False,'items':[{'tax_code':'3010101020203000000','quantity':100000000,'total_amount':'429900','discount':False}]}]
    :param scene: 场景值，目前只支持WITH_WECHATPAY。示例值：'WITH_WECHATPAY'
    """
    params = {}
    if fapiao_apply_id:
        params.update({'fapiao_apply_id': fapiao_apply_id})
    else:
        raise Exception('fapiao_aply_id is not assigned.')
    cipher_data = False
    if buyer_information:
        if buyer_information.get('phone'):
            buyer_information.update({'phone': self._core.encrypt(buyer_information.get('phone'))})
            cipher_data = True
        if buyer_information.get('email'):
            buyer_information.update({'email': self._core.encrypt(buyer_information.get('email'))})
            cipher_data = True
        params.update({'buyer_information': buyer_information})
    else:
        raise Exception('buyer_information is not assigned.')
    if fapiao_information:
        params.update({'fapiao_information': fapiao_information})
    else:
        raise Exception('fapiao_information is not assigned.')
    params.update({'scene': scene})
    path = '/v3/new-tax-control-fapiao/fapiao-applications'
    return self._core.request(path, method=RequestType.POST, data=params, cipher_data=cipher_data)


def fapiao_query(self, fapiao_apply_id, fapiao_id=None):
    """查询电子发票
    :param fapiao_apply_id: 发票申请单号，示例值：'4200000444201910177461284488'
    :param fapiao_id: 商户发票单号，示例值：'20200701123456'
    """
    path = '/v3/new-tax-control-fapiao/fapiao-applications/%s' % fapiao_apply_id
    if fapiao_id:
        path += '?fapiao_id=%s' % fapiao_id
    return self._core.request(path)


def fapiao_reverse(self, fapiao_apply_id, reverse_reason, fapiao_information):
    """冲红电子发票
    :param fapiao_apply_id: 发票申请单号，示例值：'4200000444201910177461284488'
    :param reverse_reason: 冲红原因，示例值：'退款'
    :param fapiao_information: 需要冲红的发票信息，示例值：{'fapiao_id':'20200701123456','fapiao_code':'044001911211','fapiao_number':'12897794'}
    """
    if fapiao_apply_id:
        path = '/v3/new-tax-control-fapiao/fapiao-applications/%s/reverse' % fapiao_apply_id
    else:
        raise Exception('fapiao_apply_id is not assigned.')
    params = {}
    if reverse_reason:
        params.update({'reverse_reason': reverse_reason})
    else:
        raise Exception('reverse_reason is not assigned.')
    if fapiao_information:
        params.update({'fapiao_information': fapiao_information})
    else:
        raise Exception('fapiao_information is not assigned.')
    return self._core.request(path, method=RequestType.POST, data=params)


def fapiao_upload_file(self, filepath):
    """上传电子发票文件
    :filepath: 电子发票文件路径，只支持pdf和odf两种格式，示例值：'./fapiao/0001.pdf'
    """
    if not (filepath and os.path.exists(filepath) and os.path.isfile(filepath)):
        raise Exception('filepath is not assigned or not exists')
    with open(filepath, mode='rb') as f:
        content = f.read()
    filename = os.path.basename(filepath)
    filetype = os.path.splitext(filename)[-1][1:].upper()
    mimes = {
        'PDF': 'application/pdf',
        'ODF': 'application/odf'
    }
    if filetype not in mimes:
        raise Exception('wechatpayv3 does not support this file type.')
    params = {}
    params.update({'meta': '{"file_type":"%s","digest_alogrithm":"SM3","digest":"%s"}' % (filetype, sm3(content))})
    files = [('file', (filename, content, mimes[filetype]))]
    path = '/v3/new-tax-control-fapiao/fapiao-applications/upload-fapiao-file'
    return self._core.request(path, method=RequestType.POST, data=params, sign_data=params.get('meta'), files=files)


def fapiao_insert_cards(self, fapiao_apply_id, buyer_information, fapiao_card_information, scene='WITH_WECHATPAY'):
    """将电子发票插入微信用户卡包
    :param fapiao_apply_id: 发票申请单号，示例值：'4200000444201910177461284488'
    :param buyer_information: 购买方信息，即发票抬头。示例值：{'type':'ORGANIZATION','name':'深圳市南山区测试企业'}
    :param fapiao_card_information: 电子发票卡券信息列表，最多五条。示例值：[{'fapiao_media_id':'ASNFZ4mrze/+3LqYdlQyEA==','fapiao_number':'123456','fapiao_code':'044001911211','fapiao_time':'2020-07-01T12:00:00+08:00','check_code':'69001808340631374774'......}]
    :param scene: 场景值，目前只支持WITH_WECHATPAY。示例值：'WITH_WECHATPAY'
    """
    if fapiao_apply_id:
        path = '/v3/new-tax-control-fapiao/fapiao-applications/%s/insert-cards' % fapiao_apply_id
    else:
        raise Exception('fapiao_apply_id is not assigned.')
    params = {}
    if buyer_information:
        params.update({'buyer_information': buyer_information})
    else:
        raise Exception('buyer_information is not assigned.')
    if fapiao_card_information:
        params.update({'fapiao_card_information': fapiao_card_information})
    else:
        raise Exception('fapiao_card_information is not assigned.')
    params.update({'scene': scene})
    return self._core.request(path, method=RequestType.POST, data=params)



# -*- coding: utf-8 -*-

import json
import os
from datetime import datetime

import requests

from .type import RequestType, SignType
from .utils import (aes_decrypt, build_authorization, hmac_sign,
                    load_certificate, load_private_key, rsa_decrypt,
                    rsa_encrypt, rsa_sign, rsa_verify)


class Core():
    def __init__(self, mchid, cert_serial_no, private_key, apiv3_key, cert_dir=None, logger=None, proxy=None):
        self._proxy = proxy
        self._mchid = mchid
        self._cert_serial_no = cert_serial_no
        self._private_key = load_private_key(private_key)
        self._apiv3_key = apiv3_key
        self._gate_way = 'https://api.mch.weixin.qq.com'
        self._certificates = []
        self._cert_dir = cert_dir + '/' if cert_dir else None
        self._logger = logger
        self._init_certificates()

    def _update_certificates(self):
        path = '/v3/certificates'
        self._certificates.clear()
        code, message = self.request(path, skip_verify=True)
        if code != 200:
            return
        data = json.loads(message).get('data')
        for value in data:
            serial_no = value.get('serial_no')
            effective_time = value.get('effective_time')
            expire_time = value.get('expire_time')
            encrypt_certificate = value.get('encrypt_certificate')
            algorithm = nonce = associated_data = ciphertext = None
            if encrypt_certificate:
                algorithm = encrypt_certificate.get('algorithm')
                nonce = encrypt_certificate.get('nonce')
                associated_data = encrypt_certificate.get('associated_data')
                ciphertext = encrypt_certificate.get('ciphertext')
            if not (serial_no and effective_time and expire_time and algorithm and nonce and associated_data and ciphertext):
                continue
            cert_str = aes_decrypt(
                nonce=nonce,
                ciphertext=ciphertext,
                associated_data=associated_data,
                apiv3_key=self._apiv3_key)
            certificate = load_certificate(cert_str)
            if not certificate:
                continue
            now = datetime.utcnow()
            if now < certificate.not_valid_before or now > certificate.not_valid_after:
                continue
            self._certificates.append(certificate)
            if not self._cert_dir:
                continue
            if not os.path.exists(self._cert_dir):
                os.makedirs(self._cert_dir)
            if not os.path.exists(self._cert_dir + serial_no + '.pem'):
                with open(self._cert_dir + serial_no + '.pem', 'w') as f:
                    f.write(cert_str)

    def _verify_signature(self, headers, body):
        signature = headers.get('Wechatpay-Signature')
        timestamp = headers.get('Wechatpay-Timestamp')
        nonce = headers.get('Wechatpay-Nonce')
        serial_no = headers.get('Wechatpay-Serial')
        cert_found = False
        for cert in self._certificates:
            if int('0x' + serial_no, 16) == cert.serial_number:
                cert_found = True
                certificate = cert
                break
        if not cert_found:
            self._update_certificates()
            for cert in self._certificates:
                if int('0x' + serial_no, 16) == cert.serial_number:
                    cert_found = True
                    certificate = cert
                    break
            if not cert_found:
                return False
        if not rsa_verify(timestamp, nonce, body, signature, certificate):
            return False
        return True

    def request(self, path, method=RequestType.GET, data=None, skip_verify=False, sign_data=None, files=None, cipher_data=False, headers={}):
        if files:
            headers.update({'Content-Type': 'multipart/form-data'})
        else:
            headers.update({'Content-Type': 'application/json'})
        headers.update({'Accept': 'application/json'})
        headers.update({'User-Agent': 'wechatpay v3 api python sdk(https://github.com/minibear2021/wechatpayv3)'})
        if cipher_data:
            headers.update({'Wechatpay-Serial': hex(self._last_certificate().serial_number)[2:].upper()})
        authorization = build_authorization(
            path,
            method.value,
            self._mchid,
            self._cert_serial_no,
            self._private_key,
            data=sign_data if sign_data else data)
        headers.update({'Authorization': authorization})
        if self._logger:
            self._logger.debug('Request url: %s' % self._gate_way + path)
            self._logger.debug('Request type: %s' % method.value)
            self._logger.debug('Request headers: %s' % headers)
            self._logger.debug('Request params: %s' % data)
        if method == RequestType.GET:
            response = requests.get(url=self._gate_way + path, headers=headers, proxies=self._proxy)
        elif method == RequestType.POST:
            response = requests.post(url=self._gate_way + path, json=None if files else data, data=data if files else None, headers=headers, files=files, proxies=self._proxy)
        elif method == RequestType.PATCH:
            response = requests.patch(url=self._gate_way + path, json=data, headers=headers, proxies=self._proxy)
        elif method == RequestType.PUT:
            response = requests.put(url=self._gate_way + path, json=data, headers=headers, proxies=self._proxy)
        elif method == RequestType.DELETE:
            response = requests.delete(url=self._gate_way + path, headers=headers, proxies=self._proxy)
        else:
            raise Exception('wechatpayv3 does no support this request type.')
        if self._logger:
            self._logger.debug('Response status code: %s' % response.status_code)
            self._logger.debug('Response headers: %s' % response.headers)
            self._logger.debug('Response content: %s' % response.text)
        if response.status_code in range(200, 300) and not skip_verify:
            if not self._verify_signature(response.headers, response.text):
                raise Exception('failed to verify the signature')
        return response.status_code, response.text if 'application/json' in response.headers.get('Content-Type') else response.content

    def sign(self, data, sign_type=SignType.RSA_SHA256):
        if sign_type == SignType.RSA_SHA256:
            sign_str = '\n'.join(data) + '\n'
            return rsa_sign(self._private_key, sign_str)
        elif sign_type == SignType.HMAC_SHA256:
            key_list = sorted(data.keys())
            sign_str = ''
            for k in key_list:
                v = data[k]
                sign_str += str(k) + '=' + str(v) + '&'
            sign_str += 'key=' + self._apiv3_key
            return hmac_sign(self._apiv3_key, sign_str)
        else:
            raise ValueError('unexpected value of sign_type.')

    def decrypt_callback(self, headers, body):
        if isinstance(body, bytes):
            body = body.decode('UTF-8')
        if self._logger:
            self._logger.debug('Callback Header: %s' % headers)
            self._logger.debug('Callback Body: %s' % body)
        if not self._verify_signature(headers, body):
            return None
        data = json.loads(body)
        resource_type = data.get('resource_type')
        if resource_type != 'encrypt-resource':
            return None
        resource = data.get('resource')
        if not resource:
            return None
        algorithm = resource.get('algorithm')
        if algorithm != 'AEAD_AES_256_GCM':
            raise Exception('wechatpayv3 does not support this algorithm')
        nonce = resource.get('nonce')
        ciphertext = resource.get('ciphertext')
        associated_data = resource.get('associated_data')
        if not (nonce and ciphertext):
            return None
        if not associated_data:
            associated_data = ''
        result = aes_decrypt(
            nonce=nonce,
            ciphertext=ciphertext,
            associated_data=associated_data,
            apiv3_key=self._apiv3_key)
        if self._logger:
            self._logger.debug('Callback resource: %s' % result)
        return result

    def callback(self, headers, body):
        if isinstance(body, bytes):
            body = body.decode('UTF-8')
        result = self.decrypt_callback(headers=headers, body=body)
        if result:
            data = json.loads(body)
            data.update({'resource': json.loads(result)})
            return data
        else:
            return result

    def _init_certificates(self):
        if self._cert_dir and os.path.exists(self._cert_dir):
            for file_name in os.listdir(self._cert_dir):
                if not file_name.lower().endswith('.pem'):
                    continue
                with open(self._cert_dir + file_name, encoding="utf-8") as f:
                    certificate = load_certificate(f.read())
                now = datetime.utcnow()
                if certificate and now >= certificate.not_valid_before and now <= certificate.not_valid_after:
                    self._certificates.append(certificate)
        if not self._certificates:
            self._update_certificates()
        if not self._certificates:
            raise Exception('No wechatpay platform certificate, please double check your init params.')

    def decrypt(self, ciphtext):
        return rsa_decrypt(ciphertext=ciphtext, private_key=self._private_key)

    def encrypt(self, text):
        return rsa_encrypt(text=text, certificate=self._last_certificate())

    def _last_certificate(self):
        if not self._certificates:
            self._update_certificates()
        certificate = self._certificates[0]
        for cert in self._certificates:
            if certificate.not_valid_after < cert.not_valid_after:
                certificate = cert
        return certificate



# -*- coding: utf-8 -*-

from .type import RequestType
from .transaction import query_refund, refund


def payscore_direct_complete(self, out_order_no, openid, service_id, service_introduction, post_payments,
                             time_range, total_amount, post_discounts=None, location=None,
                             profit_sharing=False, goods_tag=None, attach=None, notify_url=None, appid=None):
    """创单结单合并
    :param out_order_no: 商户服务订单号，商户系统内部服务订单号（不是交易单号），要求此参数只能由数字、大小写字母_-|*组成，且在同一个商户号下唯一。示例值:'1234323JKHDFE1243252'
    :param openid: 用户标识，微信用户在商户对应appid下的唯一标识。示例值:'oUpF8uMuAJO_M2pxb1Q9zNjWeS6o'
    :param service_id: 服务ID。示例值:'500001'
    :param service_introduction: 服务信息，用于介绍本订单所提供的服务 ，当参数长度超过20个字符时，报错处理。示例值:'某某酒店'
    :param post_payments: 付费项目列表，最多包含100条付费项目。
    :param time_range: 服务时间范围。
    :param total_amount: 总金额，总金额 =（完结付费项目1…+完结付费项目n）-（完结商户优惠项目1…+完结商户优惠项目n）。示例值:50000
    :param post_discounts: 商户优惠，付费商户优惠列表，最多包含30条商户优惠。
    :param location: 服务位置，如果传入，用户侧则显示此参数。
    :param profit_sharing: 微信支付服务分账标记，默认为false，枚举值:False:不分账，True:分账。示例值:False
    :param goods_tag: 订单优惠标记。示例值:'goods_tag1'
    :param attach: 商户数据包。商户数据包可存放本订单所需信息，需要先urlencode后传入。当商户数据包总长度超出256字符时，报错处理。示例值:'Easdfowealsdkjfnlaksjdlfkwqoi&wl3l2sald'
    :param notify_url: 商户回调地址，商户接收扣款成功回调通知的地址，服务需要收款时此参数必填；服务无需收款时此参数不填。示例值:'https://api.test.com'
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    """
    params = {}
    if not (out_order_no and openid and service_id and service_introduction and post_payments and time_range and total_amount):
        raise Exception('ut_order_no or openid or service_id or service_introduction or post_payments or time_range or total_amount is not assigned.')
    params.update({'appid': appid or self._appid})
    params.update({'out_order_no': out_order_no})
    params.update({'openid': openid})
    params.update({'service_id': service_id})
    params.update({'service_introduction': service_introduction})
    params.update({'post_payments': post_payments})
    params.update({'time_range': time_range})
    params.update({'total_amount': total_amount})
    if post_discounts:
        params.update({'post_discounts': post_discounts})
    if location:
        params.update({'location': location})
    if profit_sharing:
        params.update({'profit_sharing': profit_sharing})
    if goods_tag:
        params.update({'goods_tag': goods_tag})
    if attach:
        params.update({'attach': attach})
    payment = False
    for item in post_payments:
        if item.get('amount') > 0:
            payment = True
            break
    if payment:
        if not (notify_url or self._notify_url):
            raise Exception('notify_url is not assigned.')
        params.update({'notify_url': notify_url or self._notify_url})
    path = '/payscore/serviceorder/direct-complete'
    return self._core.request(path, method=RequestType.POST, data=params)


def payscore_permission(self, service_id, authorization_code, notify_url=None, appid=None):
    """商户预授权
    :param service_id: 服务ID。示例值:'500001'
    :param authorization_code: 授权协议号，户系统内部授权协议号，要求此参数只能由数字、大小写字母_-*组成，且在同一个商户号下唯一。示例值:'1234323JKHDFE1243252'
    :param notify_url: 通知地址，商户接收授权回调通知的地址。示例值:'http://www.qq.com'
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    """
    params = {}
    if not (service_id and authorization_code):
        raise Exception('service_id or authorization_code is not assigned.')
    params.update({'appid': appid or self._appid})
    params.update({'service_id': service_id})
    params.update({'authorization_code': authorization_code})
    params.update({'notify_url': notify_url or self._notify_url})
    path = '/v3/payscore/permissions'
    return self._core.request(path, method=RequestType.POST, data=params)


def payscore_permission_query(self, service_id, authorization_code=None, openid=None):
    """查询用户授权记录（授权协议号或openid）
    :param service_id: 服务ID。示例值:'500001'
    :param authorization_code: 授权协议号，户系统内部授权协议号，要求此参数只能由数字、大小写字母_-*组成，且在同一个商户号下唯一。示例值:'1234323JKHDFE1243252'
    :param openid: 用户标识，微信用户在商户对应appid下的唯一标识。示例值:'oUpF8uMuAJO_M2pxb1Q9zNjWeS6o'
    """
    if not service_id:
        raise Exception('service_id is not assigned.')
    if authorization_code:
        path = '/v3/payscore/permissions/authorization-code/%s?service_id=%s' % (authorization_code, service_id)
    elif openid:
        path = '/v3/payscore/permissions/openid/%s?appid=%s&service_id=%s' % (openid, self._appid, service_id)
    else:
        raise Exception('authorization_code or openid is not assigned.')
    return self._core.request(path)


def payscore_permission_terminate(self, service_id, reason, authorization_code=None, openid=None, appid=None):
    """解除用户授权记录（授权协议号或openid）
    :param service_id: 服务ID。示例值:'500001'
    :param reason: 撤销原因，解除授权原因。示例值:'撤销原因'
    :param authorization_code: 授权协议号，户系统内部授权协议号，要求此参数只能由数字、大小写字母_-*组成，且在同一个商户号下唯一。示例值:'1234323JKHDFE1243252'
    :param openid: 用户标识，微信用户在商户对应appid下的唯一标识。示例值:'oUpF8uMuAJO_M2pxb1Q9zNjWeS6o'
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    """
    params = {}
    if not (service_id and reason):
        raise Exception('service_id or reason is not assigned.')
    params.update({'service_id': service_id})
    params.update({'reason': reason})
    if authorization_code:
        path = 'v3/payscore/permissions/authorization-code/%s/terminate' % authorization_code
    elif openid:
        params.update({'appid': appid or self._appid})
        path = '/v3/payscore/permissions/openid/%s/terminate' % openid
    else:
        raise Exception('authorization_code or openid is not assigned.')
    return self._core.request(path, method=RequestType.POST, data=params)


def payscore_create(self, out_order_no, service_id, service_introduction, time_range,
                    risk_fund, attach=None, openid=None, post_payments=None, post_discounts=None,
                    location=None, need_user_confirm=True, notify_url=None, appid=None):
    """创建支付分订单
    :param out_order_no: 商户服务订单号，商户系统内部服务订单号（不是交易单号），要求此参数只能由数字、大小写字母_-|*组成，且在同一个商户号下唯一。示例值:'1234323JKHDFE1243252'
    :param service_id: 服务ID，该服务ID有本接口对应产品的权限。示例值:'500001'
    :param service_introduction: 服务信息，用于介绍本订单所提供的服务 ，当参数长度超过20个字符时，报错处理。示例值:'某某酒店'
    :param time_range: 服务时间范围。
    :param risk_fund: 订单风险金。
    :param attach: 商户数据包，商户数据包可存放本订单所需信息，需要先urlencode后传入。当商户数据包总长度超出256字符时，报错处理。示例值:'Easdfowealsdkjfnlaksjdlfkwqoi&wl3l2sald'
    :param openid: 用户标识，微信用户在商户对应appid下的唯一标识。免确认订单:必填，需确认订单:不填。示例值:'oUpF8uMuAJO_M2pxb1Q9zNjWeS6o'
    :param post_payments: 后付费项目，后付费项目列表，最多包含100条付费项目。如果传入，用户侧则显示此参数。
    :param post_discounts: 后付费商户优惠，后付费商户优惠列表，最多包含30条商户优惠。如果传入，用户侧则显示此参数。
    :param location: 服务位置信息，如果传入，用户侧则显示此参数。
    :param need_user_confirm: 是否需要用户确认，枚举值:False:免确认订单，True:需确认订单，默认值True。示例值:True
    :param notify_url: 通知地址，示例值:'https://www.weixin.qq.com/wxpay/pay.php'
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    """
    params = {}
    if out_order_no:
        params.update({'out_order_no': out_order_no})
    else:
        raise Exception('out_order_no is not assigned.')
    params.update({'appid': appid or self._appid})
    if service_id:
        params.update({'service_id': service_id})
    else:
        raise Exception('service_id is not assigned.')
    if service_introduction:
        params.update({'service_introduction': service_introduction})
    else:
        raise Exception('service_introduction is not assigned.')
    if time_range:
        params.update({'time_range': time_range})
    else:
        raise Exception('time_range is not assigned.')
    if risk_fund:
        params.update({'risk_fund': risk_fund})
    else:
        raise Exception('risk_fund is not assigned.')
    if attach:
        params.update({'attach': attach})
    if post_payments:
        params.update({'post_payments': post_payments})
    if post_discounts:
        params.update({'post_discounts': post_discounts})
    if location:
        params.update({'location': location})
    params.update({'need_user_confirm': need_user_confirm})
    if not need_user_confirm:
        if openid:
            params.update({'openid': openid})
        else:
            raise Exception('openid is not assigned.')
    params.update({'notify_url': notify_url or self._notify_url})
    path = '/v3/payscore/serviceorder'
    return self._core.request(path, method=RequestType.POST, data=params)


def payscore_query(self, service_id, out_order_no=None, query_id=None):
    """查询支付分订单
    :param service_id: 服务ID，该服务ID有本接口对应产品的权限。示例值:'500001'
    :param out_order_no: 商户服务订单号，商户系统内部服务订单号（不是交易单号），要求此参数只能由数字、大小写字母_-|*组成，且在同一个商户号下唯一。示例值:'1234323JKHDFE1243252'
    :param query_id: 回跳查询ID，微信侧回跳到商户前端时用于查单的单据查询id。商户单号与回跳查询id必填其中一个。不允许都填写或都不填写。示例值:'15646546545165651651'
    """
    if service_id:
        path = '/v3/payscore/serviceorder?service_id=%s&appid=%s' % (service_id, self._appid)
    else:
        raise Exception('service_id is not assigned.')
    if out_order_no:
        path = '%s&out_order_no=%s' % (path, out_order_no)
    elif query_id:
        path = '%s&query_id=%s' % (path, query_id)
    else:
        raise Exception('out_order_no or query_id is not assigned.')
    return self._core.request(path)


def payscore_cancel(self, out_order_no, service_id, reason, appid=None):
    """取消支付分订单
    :param out_order_no: 商户服务订单号，商户系统内部服务订单号（不是交易单号），要求此参数只能由数字、大小写字母_-|*组成，且在同一个商户号下唯一。示例值:'1234323JKHDFE1243252'
    :param service_id: 服务ID，该服务ID有本接口对应产品的权限。示例值:'500001'
    :param query_id: 回跳查询ID，微信侧回跳到商户前端时用于查单的单据查询id。商户单号与回跳查询id必填其中一个。不允许都填写或都不填写。示例值:'15646546545165651651'
    :param reason: 取消原因，最多30个字符，每个汉字/数字/英语都按1个字符计算超过长度报错处理。注:重录时需保证参数完全一致，包括取消原因。示例值:'用户投诉'
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    """
    params = {}
    if out_order_no:
        path = '/v3/payscore/serviceorder/%s/cancel' % out_order_no
    else:
        raise Exception('out_order_no is not assigned.')
    if service_id:
        params.update({'service_id': service_id})
    else:
        raise Exception('service_id is not assigned.')
    if reason:
        params.update({'reason': reason})
    else:
        raise Exception('reason is not assigned.')
    params.update({'appid': appid or self._appid})
    return self._core.request(path, method=RequestType.POST, data=params)


def payscore_modify(self, out_order_no, service_id, post_payments, total_amount, reason, post_discounts=None, appid=None):
    """修改订单金额
    :param out_order_no: 商户服务订单号，商户系统内部服务订单号（不是交易单号），要求此参数只能由数字、大小写字母_-|*组成，且在同一个商户号下唯一。示例值:'1234323JKHDFE1243252'
    :param service_id: 服务ID，该服务ID有本接口对应产品的权限。示例值:'500001'
    :param post_payments: 后付费项目，后付费项目列表，最多包含100条付费项目。
    :param total_amount: 总金额，单位为分，不能超过完结订单时候的总金额，只能为整数，详见支付金额。示例值:50000
    :param reason: 取消原因，最多30个字符，每个汉字/数字/英语都按1个字符计算超过长度报错处理。注:重录时需保证参数完全一致，包括取消原因。示例值:'用户投诉'
    :param post_discounts: 后付费商户优惠，后付费商户优惠列表，最多包含30条商户优惠。如果传入，用户侧则显示此参数。
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    """
    params = {}
    if out_order_no:
        path = '/v3/payscore/serviceorder/%s/modify' % out_order_no
    else:
        raise Exception('out_order_no is not assigned.')
    if service_id:
        params.update({'service_id': service_id})
    else:
        raise Exception('service_id is not assigned.')
    if post_payments:
        params.update({'post_payments': post_payments})
    else:
        raise Exception('post_payments is not assigned.')
    if total_amount:
        params.update({'total_amount': total_amount})
    else:
        raise Exception('total_amount is not assigned.')
    if reason:
        params.update({'reason': reason})
    else:
        raise Exception('reason is not assigned.')
    if post_discounts:
        params.update({'post_discounts': post_discounts})
    params.update({'appid': appid or self._appid})
    return self._core.request(path, method=RequestType.POST, data=params)


def payscore_complete(self, out_order_no, service_id, post_payments, total_amount, post_discounts=None,
                      time_range=None, location=None, profit_sharing=False, goods_tag=None, appid=None):
    """完结支付分订单
    :param out_order_no: 商户服务订单号，商户系统内部服务订单号（不是交易单号），要求此参数只能由数字、大小写字母_-|*组成，且在同一个商户号下唯一。示例值:'1234323JKHDFE1243252'
    :param service_id: 服务ID，该服务ID有本接口对应产品的权限。示例值:'500001'
    :param post_payments: 后付费项目，后付费项目列表，最多包含100条付费项目。如果传入，用户侧则显示此参数。
    :param total_amount: 总金额，数字，必须≥0（单位:分），只能为整数。示例值:100
    :param post_discounts: 后付费商户优惠，后付费商户优惠列表，最多包含30条商户优惠。如果传入，用户侧则显示此参数。
    :param time_range: 服务时间范围。
    :param location: 服务位置信息，如果传入，用户侧则显示此参数。
    :param profit_sharing: 微信支付服务分账标记，完结订单分账接口标记。False:不分账，True:分账，默认:False，示例值:False
    :param goods_tag: 订单优惠标记，订单优惠标记，代金券或立减金优惠的参数，示例值:'goods_tag'
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    """
    params = {}
    if out_order_no:
        path = '/v3/payscore/serviceorder/%s/complete' % out_order_no
    else:
        raise Exception('out_order_no is not assigned.')
    if service_id:
        params.update({'service_id': service_id})
    else:
        raise Exception('service_id is not assigned.')
    if post_payments:
        params.update({'post_payments': post_payments})
    else:
        raise Exception('post_payments is not assigned.')
    if type(total_amount) is int and total_amount >= 0:
        params.update({'total_amount': total_amount})
    else:
        raise Exception('total_amount is not assigned.')
    if post_discounts:
        params.update({'post_discounts': post_discounts})
    if time_range:
        params.update({'time_range': time_range})
    if location:
        params.update({'location': location})
    if goods_tag:
        params.update({'goods_tag': goods_tag})
    params.update({'profit_sharing': profit_sharing})
    params.update({'appid': appid or self._appid})
    return self._core.request(path, method=RequestType.POST, data=params)


def payscore_pay(self, out_order_no, service_id, appid=None):
    """商户发起催收扣款
    :param out_order_no: 商户服务订单号，商户系统内部服务订单号（不是交易单号），要求此参数只能由数字、大小写字母_-|*组成，且在同一个商户号下唯一。示例值:'1234323JKHDFE1243252'
    :param service_id: 服务ID，该服务ID有本接口对应产品的权限。示例值:'500001'
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    """
    params = {}
    if out_order_no:
        path = '/v3/payscore/serviceorder/%s/pay' % out_order_no
    else:
        raise Exception('out_order_no is not assigned.')
    if service_id:
        params.update({'service_id': service_id})
    else:
        raise Exception('service_id is not assigned.')
    params.update({'appid': appid or self._appid})
    return self._core.request(path, method=RequestType.POST, data=params)


def payscore_sync(self, out_order_no, service_id, scene_type='Order_Paid', detail={'paid_time': None}, appid=None):
    """同步服务订单信息
    :param out_order_no: 商户服务订单号，商户系统内部服务订单号（不是交易单号），要求此参数只能由数字、大小写字母_-|*组成，且在同一个商户号下唯一。示例值:'1234323JKHDFE1243252'
    :param service_id: 服务ID，该服务ID有本接口对应产品的权限。示例值:'500001'
    :param scene_type: 场景类型，场景类型为“Order_Paid”，表示“订单收款成功” 。示例值:'Order_Paid'
    :param detail: 内容信息详情，场景类型为Order_Paid时，为必填项。其中	paid_time表示收款成功时间，示例值:'20091225091210'
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    """
    params = {}
    if out_order_no:
        path = '/v3/payscore/serviceorder/%s/sync' % out_order_no
    else:
        raise Exception('out_order_no is not assigned.')
    if service_id:
        params.update({'service_id': service_id})
    else:
        raise Exception('service_id is not assigned.')
    if scene_type:
        params.update({'type': scene_type})
    else:
        raise Exception('scene_type is not assigned.')
    if detail:
        params.update({'detail': detail})
    else:
        raise Exception('detail is not assigned.')
    params.update({'appid': appid or self._appid})
    return self._core.request(path, method=RequestType.POST, data=params)


def payscore_refund(self, transaction_id, out_refund_no, amount, reason=None,
                    funds_account=None, goods_detail=None, notify_url=None):
    """申请退款
    :param transaction_id: 微信支付订单号，示例值:'1217752501201407033233368018'
    :param out_refund_no: 商户退款单号，示例值:'1217752501201407033233368018'
    :param amount: 金额信息，示例值:{'refund':888, 'total':888, 'currency':'CNY'}
    :param reason: 退款原因，示例值:'商品已售完'
    :param funds_account: 退款资金来源，示例值:'AVAILABLE'
    :param goods_detail: 退款商品，示例值:{'merchant_goods_id':'1217752501201407033233368018', 'wechatpay_goods_id':'1001', 'goods_name':'iPhone6s 16G', 'unit_price':528800, 'refund_amount':528800, 'refund_quantity':1}
    :param notify_url: 通知地址，示例值:'https://www.weixin.qq.com/wxpay/pay.php'
    """
    return refund(self, out_refund_no=out_refund_no, amount=amount, transaction_id=transaction_id, reason=reason,
                  funds_account=funds_account, goods_detail=goods_detail, notify_url=notify_url)


def payscore_refund_query(self, out_refund_no):
    """查询单笔退款
    :param out_refund_no: 商户退款单号，示例值:'1217752501201407033233368018'
    """
    return query_refund(self, out_refund_no=out_refund_no)


def payscore_merchant_bill(self, bill_date, service_id, tar_type='GZIP', encryption_algorithm='AEAD_AES_256_GCM'):
    """商户申请获取对账单
    :param bill_date: 账单日期，格式'YYYY-MM-DD'，仅支持下载近三个月的账单。示例值:'2021-01-01'
    :param service_id: 支付分服务ID。示例值:'2002000000000558128851361561536'
    :param tar_type: 账单的压缩类型，'GZIP':文件压缩方式为gzip，返回.gzip格式的压缩文件。示例值:'GZIP'
    :param encryption_algorithm: 加密算法，对返回账单原文加密的算法'AEAD_AES_256_GCM'，账单使用AEAD_AES_256_GCM加密算法进行加密。示例值:'AEAD_AES_256_GCM'
    """
    if bill_date:
        path = '/v3/payscore/merchant-bill?bill_date=%s' % bill_date
    else:
        raise Exception('bill_date is not assigned.')
    if service_id:
        path = '%s&service_id=%s' % (path, service_id)
    else:
        raise Exception('service_id is not assigned.')
    path = '%s&tar_type=%s' % (path, tar_type if tar_type else 'GZIP')
    path = '%s&encryption_algorithm=%s' % (path, encryption_algorithm if encryption_algorithm else 'AEAD_AES_256_GCM')
    return self._core.request(path)
